#include <iostream>
#include <map>
#include <string>
#include <cstring>
#include <mysql++/mysql++.h>
#include <sys/utsname.h>

#include <boost/extension/extension.hpp>
#include <boost/extension/factory.hpp>
#include <boost/extension/type_map.hpp>
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/xml_parser.hpp>

#include <plugins/plugin.hpp>
#include <syslog.h>

#include <ndds/ndds_cpp.h>
#include "net_load_dds.h"
#include "net_load_ddsSupport.h"

#ifndef SYSCONFDIR
#  define SYSCONFDIR ""
#endif

class netloadListener : public DDSDataReaderListener {
  public:
  netloadListener(std::string plugin_id);
  ~netloadListener(void);
  
  virtual void on_requested_deadline_missed(DDSDataReader* /*reader*/,
					    const DDS_RequestedDeadlineMissedStatus& /*status*/) {}
    
  virtual void on_requested_incompatible_qos(DDSDataReader* /*reader*/,
					     const DDS_RequestedIncompatibleQosStatus& /*status*/) {}
  
  virtual void on_sample_rejected(DDSDataReader* /*reader*/,
				  const DDS_SampleRejectedStatus& /*status*/) {}
  
  virtual void on_liveliness_changed(DDSDataReader* /*reader*/,
				     const DDS_LivelinessChangedStatus& /*status*/) {}
  
  virtual void on_sample_lost(DDSDataReader* /*reader*/,
			      const DDS_SampleLostStatus& /*status*/) {}
  
  virtual void on_subscription_matched(DDSDataReader* /*reader*/,
				       const DDS_SubscriptionMatchedStatus& /*status*/) {}
  
  virtual void on_data_available(DDSDataReader* reader);
  
private:
  std::map<std::string,std::string> hostinfo_map_;  
  std::map<std::string,int> rowcount_map_;
  mysqlpp::Connection conn_;
  std::string database_server_;
  std::string database_name_;
  std::string database_user_;
  std::string database_password_;
  int rows_;
  std::string plugin_id_;
  
};


class net_load : public plugin {
public:
  net_load(std::string plugin_id);
  virtual void receive_information(void);
  virtual void clean_dds_stuff(void);
private:
  //DDS Stuff
  DDSDomainParticipant *participant_;
  DDSSubscriber *subscriber_;
  DDSTopic *topic_;
  DDSTopicDescription* topic_or_cft_;
  DDSDataReader *reader_;
  netloadDataReader * netload_reader_;
  netloadListener *reader_listener_; 
  DDS_ReturnCode_t retcode_;
  const char *type_name_;
  DDS_Duration_t receive_period_;
  int domain_id_;
  std::string plugin_id_;
  std::string topic_name_;
  std::string qos_library_;
  std::string qos_profile_;
  
  
};


BOOST_EXTENSION_TYPE_MAP_FUNCTION
{
  using namespace boost::extensions;
  std::map<std::string, factory<plugin, std::string> > &plugin_factories(types.get());
  plugin_factories["net_load"].set<net_load>();
}



